import textwrap

from cpl_core.console import Console

from py_to_uxf_core.model.dimension import Dimension
from py_to_uxf_core.model.position import Position
from py_to_uxf_core.model.python_class import PythonClass


class UMLClass:

    def __init__(
            self,
            cls: PythonClass,
            pos: Position,
            dim: Dimension
    ):
        self._cls = cls
        self._position = pos
        self._dimension = dim

    @property
    def cls(self) -> PythonClass:
        return self._cls

    @property
    def position(self) -> Position:
        return self._position

    @position.setter
    def position(self, value: int):
        self._position = value

    @property
    def dimension(self) -> Dimension:
        return self._dimension

    @dimension.setter
    def dimension(self, value: int):
        self._dimension = value

    def as_xml(self) -> str:
        px_per_line = 16
        px_per_char = 2.9
        self._dimension.height += (self._cls.attributes.count() + self._cls.functions.count()) * px_per_line
        longest_line_length = self._dimension.width / px_per_char

        attributes = ''
        functions = ''

        if len(self._cls.attributes) > 0:
            for atr in self._cls.attributes:
                attribute = f'{atr.access_modifier.value}{atr.name}: {atr.type}\n'
                if len(attribute) > longest_line_length:
                    longest_line_length = len(attribute)
                attributes += attribute

        if len(self._cls.functions) > 0:
            for func in self._cls.functions:
                args = ''
                function = f'{func.access_modifier.value}{func.name}({args}): {func.return_type}\n'
                if len(function) > longest_line_length:
                    longest_line_length = len(function)
                functions += function

        self._dimension.width = round(longest_line_length * px_per_char * px_per_char)

        return f"""\
            <element>
                <id>UMLClass</id>
                <coordinates>
                    <x>{self._position.x}</x>
                    <y>{self._position.y}</y>
                    <w>{self._dimension.width}</w>
                    <h>{self._dimension.height}</h>
                </coordinates>
                <panel_attributes>
                    {self._cls.name}
                    --
                    {attributes}
                    --
                    {functions}
                </panel_attributes>
                <additional_attributes></additional_attributes>
            </element>
        """.replace('    ', '').replace('\t', '')
