import traceback
from typing import Optional

from cpl_core.console import Console
from cpl_query.extension import List

from py_to_uxf_core.abc.attribute_scanner_abc import AttributeScannerABC
from py_to_uxf_core.abc.class_scanner_abc import ClassScannerABC
from py_to_uxf_core.abc.file_scanner_abc import FileScannerABC
from py_to_uxf_core.abc.function_scanner_abc import FunctionScannerABC
from py_to_uxf_core.abc.python_parser_abc import PythonParserABC
from py_to_uxf_core.model.python_class import PythonClass


class PythonParserService(PythonParserABC):

    def __init__(
            self,
            file_scanner: FileScannerABC,
            class_scanner: ClassScannerABC,
            function_scanner: FunctionScannerABC,
            attribute_scanner: AttributeScannerABC
    ):
        PythonParserABC.__init__(self)

        self._file_scanner = file_scanner
        self._class_scanner = class_scanner
        self._function_scanner = function_scanner
        self._attribute_scanner = attribute_scanner

    def parse(self) -> List[PythonClass]:
        files = self._file_scanner.scan_files()
        classes = List(PythonClass)
        for file in files:
            is_comment = False
            is_function = False
            with open(file, 'r') as file_content:
                cls: Optional[PythonClass] = None
                lines = file_content.readlines()

                for i in range(len(lines)):
                    try:
                        line = lines[i]
                        line_with_tabs = line
                        line = line.replace('    ', '')
                        line = line.replace('\t', '')
                        # replace line break at the end of line
                        if line.endswith('\n'):
                            line = line.replace('\n', '')

                        if line == '\n' or line == '':
                            if is_function:
                                is_function = False
                            continue

                        # one line comments
                        if line != '"""' and line.startswith('"""') and line.endswith('"""') or line.startswith('#'):
                            continue

                        # start multi line comment
                        if line.startswith('"""') or line.count('"""') == 1 and not is_comment:
                            is_comment = True
                            continue

                        # end multi line comment
                        if line.startswith('"""') or line.endswith('"""') or line.count('"""') == 1 and is_comment:
                            is_comment = False
                            continue

                        if is_comment:
                            continue

                        if cls is None:
                            cls = self._class_scanner.scan_line_for_classes(line)
                            if cls is not None:
                                classes.append(cls)
                            continue

                        func = self._function_scanner.scan_line_for_function(line)
                        if func is not None:
                            cls.add_function(func)
                            is_function = True if func.name != '__init__' else False
                            continue

                        if len(line_with_tabs.split('    ')) > 3 or is_function:
                            continue

                        attribute = self._attribute_scanner.scan_line_for_attribute(line)
                        if attribute is not None:
                            cls.add_attribute(attribute)
                    except Exception as e:
                        Console.error(f'Parsing {file}@{i}', f'{e} -> {traceback.format_exc()}')
                        exit()

        return classes
